@@ -3,7 +3,7 @@
  * unpack.c - the huge function process_archive
  *
  * Copyright © 1995 Ian Jackson <ian@chiark.greenend.org.uk>
- * Copyright © 2006-2012 Guillem Jover <guillem@debian.org>
+ * Copyright © 2006-2014 Guillem Jover <guillem@debian.org>
  * Copyright © 2011 Linaro Limited
  * Copyright © 2011 Raphaël Hertzog <hertzog@debian.org>
  *
@@ -18,7 +18,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -45,6 +45,7 @@
 #include <dpkg/dpkg.h>
 #include <dpkg/dpkg-db.h>
 #include <dpkg/pkg.h>
+#include <dpkg/pkg-queue.h>
 #include <dpkg/path.h>
 #include <dpkg/buffer.h>
 #include <dpkg/subproc.h>
@@ -102,8 +103,8 @@ deb_reassemble(const char **filename, const char **pfilename)
     ohshite(_("unable to execute %s (%s)"),
             _("split package reassembly"), SPLITTER);
   }
-  status = subproc_wait(pid, SPLITTER);
-  switch (WIFEXITED(status) ? WEXITSTATUS(status) : -1) {
+  status = subproc_reap(pid, SPLITTER, SUBPROC_RETERROR);
+  switch (status) {
   case 0:
     /* It was a part - is it complete? */
     if (!stat(reasmbuf, &stab)) {
@@ -119,7 +120,7 @@ deb_reassemble(const char **filename, const char **pfilename)
     /* No, it wasn't a part. */
     break;
   default:
-    subproc_check(status, SPLITTER, 0);
+    internerr("unexpected exit status %d from %s", status, SPLITTER);
   }
 
   return true;
@@ -144,7 +145,7 @@ deb_verify(const char *filename)
   } else {
     int status;
 
-    status = subproc_wait(pid, "debsig-verify");
+    status = subproc_reap(pid, "debsig-verify", SUBPROC_NOCHECK);
     if (!(WIFEXITED(status) && WEXITSTATUS(status) == 0)) {
       if (!fc_badverify)
         ohshit(_("verification on package %s failed!"), filename);
@@ -196,19 +197,112 @@ get_control_dir(char *cidir)
   return cidir;
 }
 
-#define MAXCONFLICTORS 20
+/**
+ * Read the conffiles, and copy the hashes across.
+ */
+static void
+deb_parse_conffiles(struct pkginfo *pkg, const char *control_conffiles,
+                    struct filenamenode_queue *newconffiles)
+{
+  FILE *conff;
+  char conffilenamebuf[MAXCONFFILENAME];
 
-static struct pkginfo *conflictor[MAXCONFLICTORS];
-static int cflict_index = 0;
+  conff = fopen(control_conffiles, "r");
+  if (conff == NULL) {
+    if (errno == ENOENT)
+      return;
+    ohshite(_("error trying to open %.250s"), control_conffiles);
+  }
+
+  push_cleanup(cu_closestream, ehflag_bombout, NULL, 0, 1, conff);
+
+  while (fgets(conffilenamebuf, MAXCONFFILENAME - 2, conff)) {
+    struct pkginfo *otherpkg;
+    struct filepackages_iterator *iter;
+    struct filenamenode *namenode;
+    struct fileinlist *newconff;
+    struct conffile *searchconff;
+    char *p;
+
+    p = conffilenamebuf + strlen(conffilenamebuf);
+    assert(p != conffilenamebuf);
+    if (p[-1] != '\n')
+      ohshit(_("conffile name '%s' is too long, or missing final newline"),
+             conffilenamebuf);
+    while (p > conffilenamebuf && isspace(p[-1]))
+      --p;
+    if (p == conffilenamebuf)
+      continue;
+    *p = '\0';
+
+    namenode = findnamenode(conffilenamebuf, 0);
+    namenode->oldhash = NEWCONFFILEFLAG;
+    newconff = filenamenode_queue_push(newconffiles, namenode);
+
+    /*
+     * Let's see if any packages have this file.
+     *
+     * If they do we check to see if they listed it as a conffile,
+     * and if they did we copy the hash across. Since (for plain
+     * file conffiles, which is the only kind we are supposed to
+     * have) there will only be one package which ‘has’ the file,
+     * this will usually mean we only look in the package which
+     * we are installing now.
+     *
+     * The ‘conffiles’ data in the status file is ignored when a
+     * package is not also listed in the file ownership database as
+     * having that file. If several packages are listed as owning
+     * the file we pick one at random.
+     */
+    searchconff = NULL;
+
+    iter = filepackages_iter_new(newconff->namenode);
+    while ((otherpkg = filepackages_iter_next(iter))) {
+      debug(dbg_conffdetail,
+            "process_archive conffile '%s' in package %s - conff ?",
+            newconff->namenode->name, pkg_name(otherpkg, pnaw_always));
+      for (searchconff = otherpkg->installed.conffiles;
+           searchconff && strcmp(newconff->namenode->name, searchconff->name);
+           searchconff = searchconff->next)
+        debug(dbg_conffdetail,
+              "process_archive conffile '%s' in package %s - conff ? not '%s'",
+              newconff->namenode->name, pkg_name(otherpkg, pnaw_always),
+              searchconff->name);
+      if (searchconff) {
+        debug(dbg_conff,
+              "process_archive conffile '%s' package=%s %s hash=%s",
+              newconff->namenode->name, pkg_name(otherpkg, pnaw_always),
+              otherpkg == pkg ? "same" : "different!",
+              searchconff->hash);
+        if (otherpkg == pkg)
+          break;
+      }
+    }
+    filepackages_iter_free(iter);
+
+    if (searchconff) {
+      /* We don't copy ‘obsolete’; it's not obsolete in the new package. */
+      newconff->namenode->oldhash = searchconff->hash;
+    } else {
+      debug(dbg_conff, "process_archive conffile '%s' no package, no hash",
+            newconff->namenode->name);
+    }
+    newconff->namenode->flags |= fnnf_new_conff;
+  }
+
+  if (ferror(conff))
+    ohshite(_("read error in %.250s"), control_conffiles);
+  pop_cleanup(ehflag_normaltidy); /* conff = fopen() */
+  if (fclose(conff))
+    ohshite(_("error closing %.250s"), control_conffiles);
+}
+
+static struct pkg_queue conflictors = PKG_QUEUE_INIT;
 
 void
-enqueue_conflictor(struct pkginfo *pkg, struct pkginfo *pkg_fixbyrm)
+enqueue_conflictor(struct pkginfo *pkg)
 {
-  if (cflict_index >= MAXCONFLICTORS)
-    ohshit(_("package %s has too many Conflicts/Replaces pairs"),
-           pkgbin_name(pkg, &pkg->available, pnaw_nonambig));
-
-  conflictor[cflict_index++] = pkg_fixbyrm;
+  pkg_queue_push(&conflictors, pkg);
 }
 
 static void
@@ -329,8 +423,8 @@ pkg_infodb_update(struct pkginfo *pkg, char *cidir, char *cidirrest)
   /* If the old and new versions use a different infodb layout, get rid
    * of the files using the old layout. */
   if (pkg->installed.multiarch != pkg->available.multiarch &&
-      (pkg->installed.multiarch == multiarch_same ||
-       pkg->available.multiarch == multiarch_same)) {
+      (pkg->installed.multiarch == PKG_MULTIARCH_SAME ||
+       pkg->available.multiarch == PKG_MULTIARCH_SAME)) {
     debug(dbg_scripts,
           "process_archive remove old info files after db layout switch");
     pkg_infodb_foreach(pkg, &pkg->installed, pkg_infodb_remove_file);
@@ -349,22 +443,22 @@ pkg_disappear(struct pkginfo *pkg, struct pkginfo *infavour)
         pkg_name(pkg, pnaw_always));
 
   trig_activate_packageprocessing(pkg);
-  maintainer_script_installed(pkg, POSTRMFILE,
-                              "post-removal script (for disappearance)",
-                              "disappear",
-                              pkgbin_name(infavour, &infavour->available,
-                                          pnaw_nonambig),
-                              versiondescribe(&infavour->available.version,
-                                              vdew_nonambig),
-                              NULL);
+  maintscript_installed(pkg, POSTRMFILE,
+                        "post-removal script (for disappearance)",
+                        "disappear",
+                        pkgbin_name(infavour, &infavour->available,
+                                    pnaw_nonambig),
+                        versiondescribe(&infavour->available.version,
+                                        vdew_nonambig),
+                        NULL);
 
   /* OK, now we delete all the stuff in the ‘info’ directory ... */
   debug(dbg_general, "pkg_disappear cleaning info directory");
   pkg_infodb_foreach(pkg, &pkg->installed, pkg_infodb_remove_file);
   dir_sync_path(pkg_infodb_get_dir());
 
-  pkg_set_status(pkg, stat_notinstalled);
-  pkg_set_want(pkg, want_unknown);
+  pkg_set_status(pkg, PKG_STAT_NOTINSTALLED);
+  pkg_set_want(pkg, PKG_WANT_UNKNOWN);
   pkg_reset_eflags(pkg);
 
   dpkg_version_blank(&pkg->configversion);
@@ -389,7 +483,7 @@ pkgset_getting_in_sync(struct pkginfo *pkg)
   for (otherpkg = &pkg->set->pkg; otherpkg; otherpkg = otherpkg->arch_next) {
     if (otherpkg == pkg)
       continue;
-    if (otherpkg->status <= stat_configfiles)
+    if (otherpkg->status <= PKG_STAT_CONFIGFILES)
       continue;
     if (dpkg_version_compare(&pkg->available.version,
                              &otherpkg->installed.version)) {
@@ -415,30 +509,29 @@ void process_archive(const char *filename) {
    * we unwind the stack before processing the cleanup list, and these
    * variables had better still exist ... */
   static int p1[2];
-  static char *cidir = NULL;
-  static struct fileinlist *newconffiles, *newfileslist;
   static enum pkgstatus oldversionstatus;
   static struct varbuf depprobwhy;
   static struct tarcontext tc;
 
   struct dpkg_error err;
   enum parsedbflags parsedb_flags;
-  int r, i;
+  int rc;
   pid_t pid;
   struct pkgiterator *it;
   struct pkginfo *pkg, *otherpkg;
-  char *cidirrest, *p;
-  char conffilenamebuf[MAXCONFFILENAME];
+  struct pkg_list *conflictor_iter;
+  char *cidir = NULL;
+  char *cidirrest;
   char *psize;
   const char *pfilename;
-  struct fileinlist *newconff, **newconffileslastp;
+  struct fileinlist *newfileslist;
   struct fileinlist *cfile;
+  struct filenamenode_queue newconffiles;
   struct reversefilelistiter rlistit;
-  struct conffile *searchconff, **iconffileslastp, *newiconff;
+  struct conffile **iconffileslastp, *newiconff;
   struct dependency *dsearch, *newdeplist, **newdeplistlastp;
   struct dependency *newdep, *dep, *providecheck;
   struct deppossi *psearch, **newpossilastp, *possi, *newpossi, *pdep;
-  FILE *conff;
   struct filenamenode *namenode;
   struct stat stab, oldfs;
   struct pkg_deconf_list *deconpil;
@@ -472,7 +565,7 @@ void process_archive(const char *filename) {
     ohshite(_("unable to execute %s (%s)"),
             _("package control information extraction"), BACKEND);
   }
-  subproc_wait_check(pid, BACKEND " --control", 0);
+  subproc_reap(pid, BACKEND " --control", 0);
 
   /* We want to guarantee the extracted files are on the disk, so that the
    * subsequent renames to the info database do not end up with old or zero
@@ -511,18 +604,19 @@ void process_archive(const char *filename) {
     return;
   }
 
-  if (pkg->available.arch->type != arch_all &&
-      pkg->available.arch->type != arch_native &&
-      pkg->available.arch->type != arch_foreign)
+  if (pkg->available.arch->type != DPKG_ARCH_ALL &&
+      pkg->available.arch->type != DPKG_ARCH_NATIVE &&
+      pkg->available.arch->type != DPKG_ARCH_FOREIGN)
     forcibleerr(fc_architecture,
                 _("package architecture (%s) does not match system (%s)"),
-                pkg->available.arch->name, dpkg_arch_get(arch_native)->name);
+                pkg->available.arch->name,
+                dpkg_arch_get(DPKG_ARCH_NATIVE)->name);
 
   clear_deconfigure_queue();
   clear_istobes();
 
   if (wanttoinstall(pkg)) {
-    pkg_set_want(pkg, want_install);
+    pkg_set_want(pkg, PKG_WANT_INSTALL);
   } else {
       pop_cleanup(ehflag_normaltidy);
       return;
@@ -532,7 +626,7 @@ void process_archive(const char *filename) {
   for (otherpkg = &pkg->set->pkg; otherpkg; otherpkg = otherpkg->arch_next) {
     if (otherpkg == pkg)
       continue;
-    if (otherpkg->status <= stat_halfconfigured)
+    if (otherpkg->status <= PKG_STAT_HALFCONFIGURED)
       continue;
 
     if (dpkg_version_compare(&pkg->available.version,
@@ -542,7 +636,7 @@ void process_archive(const char *filename) {
 
   /* Check if anything is installed that we conflict with, or not installed
    * that we need. */
-  pkg->clientdata->istobe = itb_installnew;
+  pkg->clientdata->istobe = PKG_ISTOBE_INSTALLNEW;
 
   for (dsearch= pkg->available.depends; dsearch; dsearch= dsearch->next) {
     switch (dsearch->type) {
@@ -599,15 +693,12 @@ void process_archive(const char *filename) {
   filesdbinit();
   trig_file_interests_ensure();
 
-  if (pkg->status != stat_notinstalled && pkg->status != stat_configfiles) {
-    printf(_("Preparing to replace %s %s (using %s) ...\n"),
-           pkg_name(pkg, pnaw_nonambig),
-           versiondescribe(&pkg->installed.version,vdew_nonambig),
-           pfilename);
+  printf(_("Preparing to unpack %s ...\n"), pfilename);
+
+  if (pkg->status != PKG_STAT_NOTINSTALLED &&
+      pkg->status != PKG_STAT_CONFIGFILES) {
     log_action("upgrade", pkg, &pkg->installed);
   } else {
-    printf(_("Unpacking %s (from %s) ...\n"),
-           pkgbin_name(pkg, &pkg->available, pnaw_nonambig), pfilename);
     log_action("install", pkg, &pkg->available);
   }
 
@@ -625,112 +716,47 @@ void process_archive(const char *filename) {
   trig_parse_ci(cidir, NULL, trig_cicb_statuschange_activate, pkg, &pkg->available);
 
   /* Read the conffiles, and copy the hashes across. */
-  newconffiles = NULL;
-  newconffileslastp = &newconffiles;
+  newconffiles.head = NULL;
+  newconffiles.tail = &newconffiles.head;
   push_cleanup(cu_fileslist, ~0, NULL, 0, 0);
   strcpy(cidirrest,CONFFILESFILE);
-  conff= fopen(cidir,"r");
-  if (conff) {
-    push_cleanup(cu_closestream, ehflag_bombout, NULL, 0, 1, (void *)conff);
-    while (fgets(conffilenamebuf,MAXCONFFILENAME-2,conff)) {
-      struct filepackages_iterator *iter;
-
-      p= conffilenamebuf + strlen(conffilenamebuf);
-      assert(p != conffilenamebuf);
-      if (p[-1] != '\n')
-        ohshit(_("name of conffile (starting `%.250s') is too long (>%d characters)"),
-               conffilenamebuf, MAXCONFFILENAME);
-      while (p > conffilenamebuf && isspace(p[-1])) --p;
-      if (p == conffilenamebuf) continue;
-      *p = '\0';
-      namenode= findnamenode(conffilenamebuf, 0);
-      namenode->oldhash= NEWCONFFILEFLAG;
-      newconff= newconff_append(&newconffileslastp, namenode);
-
-      /* Let's see if any packages have this file. If they do we
-       * check to see if they listed it as a conffile, and if they did
-       * we copy the hash across. Since (for plain file conffiles,
-       * which is the only kind we are supposed to have) there will
-       * only be one package which ‘has’ the file, this will usually
-       * mean we only look in the package which we're installing now.
-       * The ‘conffiles’ data in the status file is ignored when a
-       * package isn't also listed in the file ownership database as
-       * having that file. If several packages are listed as owning
-       * the file we pick one at random. */
-      searchconff = NULL;
-
-      iter = filepackages_iter_new(newconff->namenode);
-      while ((otherpkg = filepackages_iter_next(iter))) {
-        debug(dbg_conffdetail,
-              "process_archive conffile '%s' in package %s - conff ?",
-              newconff->namenode->name, pkg_name(otherpkg, pnaw_always));
-        for (searchconff = otherpkg->installed.conffiles;
-             searchconff && strcmp(newconff->namenode->name, searchconff->name);
-             searchconff = searchconff->next)
-          debug(dbg_conffdetail,
-                "process_archive conffile '%s' in package %s - conff ? not '%s'",
-                newconff->namenode->name, pkg_name(otherpkg, pnaw_always),
-                searchconff->name);
-        if (searchconff) {
-          debug(dbg_conff,
-                "process_archive conffile '%s' package=%s %s hash=%s",
-                newconff->namenode->name, pkg_name(otherpkg, pnaw_always),
-                otherpkg == pkg ? "same" : "different!",
-                searchconff->hash);
-          if (otherpkg == pkg)
-            break;
-        }
-      }
-      filepackages_iter_free(iter);
-
-      if (searchconff) {
-        newconff->namenode->oldhash= searchconff->hash;
-	/* We don't copy ‘obsolete’; it's not obsolete in the new package. */
-      } else {
-        debug(dbg_conff, "process_archive conffile '%s' no package, no hash",
-              newconff->namenode->name);
-      }
-      newconff->namenode->flags |= fnnf_new_conff;
-    }
-    if (ferror(conff)) ohshite(_("read error in %.250s"),cidir);
-    pop_cleanup(ehflag_normaltidy); /* conff = fopen() */
-    if (fclose(conff)) ohshite(_("error closing %.250s"),cidir);
-  } else {
-    if (errno != ENOENT) ohshite(_("error trying to open %.250s"),cidir);
-  }
+  deb_parse_conffiles(pkg, cidir, &newconffiles);
 
   /* All the old conffiles are marked with a flag, so that we don't delete
    * them if they seem to disappear completely. */
   oldconffsetflags(pkg->installed.conffiles);
-  for (i = 0 ; i < cflict_index ; i++) {
-    oldconffsetflags(conflictor[i]->installed.conffiles);
-  }
+  for (conflictor_iter = conflictors.head;
+       conflictor_iter;
+       conflictor_iter = conflictor_iter->next)
+    oldconffsetflags(conflictor_iter->pkg->installed.conffiles);
 
   oldversionstatus= pkg->status;
 
-  assert(oldversionstatus <= stat_installed);
+  assert(oldversionstatus <= PKG_STAT_INSTALLED);
   debug(dbg_general,"process_archive oldversionstatus=%s",
         statusstrings[oldversionstatus]);
 
-  if (oldversionstatus == stat_halfconfigured ||
-      oldversionstatus == stat_triggersawaited ||
-      oldversionstatus == stat_triggerspending ||
-      oldversionstatus == stat_installed) {
-    pkg_set_eflags(pkg, eflag_reinstreq);
-    pkg_set_status(pkg, stat_halfconfigured);
+  if (oldversionstatus == PKG_STAT_HALFCONFIGURED ||
+      oldversionstatus == PKG_STAT_TRIGGERSAWAITED ||
+      oldversionstatus == PKG_STAT_TRIGGERSPENDING ||
+      oldversionstatus == PKG_STAT_INSTALLED) {
+    pkg_set_eflags(pkg, PKG_EFLAG_REINSTREQ);
+    pkg_set_status(pkg, PKG_STAT_HALFCONFIGURED);
     modstatdb_note(pkg);
     push_cleanup(cu_prermupgrade, ~ehflag_normaltidy, NULL, 0, 1, (void *)pkg);
     if (dpkg_version_compare(&pkg->available.version,
                              &pkg->installed.version) >= 0)
       /* Upgrade or reinstall. */
-      maintainer_script_alternative(pkg, PRERMFILE, "pre-removal", cidir, cidirrest,
-                                    "upgrade", "failed-upgrade");
+      maintscript_fallback(pkg, PRERMFILE, "pre-removal", cidir, cidirrest,
+                           "upgrade", "failed-upgrade");
     else /* Downgrade => no fallback */
-      maintainer_script_installed(pkg, PRERMFILE, "pre-removal", "upgrade",
-                                  versiondescribe(&pkg->available.version,
-                                                  vdew_nonambig), NULL);
-    pkg_set_status(pkg, stat_unpacked);
-    oldversionstatus= stat_unpacked;
+      maintscript_installed(pkg, PRERMFILE, "pre-removal",
+                            "upgrade",
+                            versiondescribe(&pkg->available.version,
+                                            vdew_nonambig),
+                            NULL);
+    pkg_set_status(pkg, PKG_STAT_UNPACKED);
+    oldversionstatus = PKG_STAT_UNPACKED;
     modstatdb_note(pkg);
   }
 
@@ -738,15 +764,18 @@ void process_archive(const char *filename) {
     struct pkginfo *removing = deconpil->pkg_removal;
 
     if (removing)
-      printf(_("De-configuring %s, to allow removal of %s ...\n"),
+      printf(_("De-configuring %s (%s), to allow removal of %s (%s) ...\n"),
              pkg_name(deconpil->pkg, pnaw_nonambig),
-             pkg_name(removing, pnaw_nonambig));
+             versiondescribe(&deconpil->pkg->installed.version, vdew_nonambig),
+             pkg_name(removing, pnaw_nonambig),
+             versiondescribe(&removing->installed.version, vdew_nonambig));
     else
-      printf(_("De-configuring %s ...\n"),
-             pkg_name(deconpil->pkg, pnaw_nonambig));
+      printf(_("De-configuring %s (%s) ...\n"),
+             pkg_name(deconpil->pkg, pnaw_nonambig),
+             versiondescribe(&deconpil->pkg->installed.version, vdew_nonambig));
 
     trig_activate_packageprocessing(deconpil->pkg);
-    pkg_set_status(deconpil->pkg, stat_halfconfigured);
+    pkg_set_status(deconpil->pkg, PKG_STAT_HALFCONFIGURED);
     modstatdb_note(deconpil->pkg);
 
     /* This means that we *either* go and run postinst abort-deconfigure,
@@ -757,74 +786,90 @@ void process_archive(const char *filename) {
                  3, (void*)deconpil->pkg, (void*)removing, (void*)pkg);
 
     if (removing) {
-      maintainer_script_installed(deconpil->pkg, PRERMFILE, "pre-removal",
-                                  "deconfigure", "in-favour",
-                                  pkgbin_name(pkg, &pkg->available, pnaw_nonambig),
-                                  versiondescribe(&pkg->available.version,
-                                                  vdew_nonambig),
-                                  "removing",
-                                  pkg_name(removing, pnaw_nonambig),
-                                  versiondescribe(&removing->installed.version,
-                                                  vdew_nonambig),
-                                  NULL);
+      maintscript_installed(deconpil->pkg, PRERMFILE, "pre-removal",
+                            "deconfigure", "in-favour",
+                            pkgbin_name(pkg, &pkg->available, pnaw_nonambig),
+                            versiondescribe(&pkg->available.version,
+                                            vdew_nonambig),
+                            "removing",
+                            pkg_name(removing, pnaw_nonambig),
+                            versiondescribe(&removing->installed.version,
+                                            vdew_nonambig),
+                            NULL);
     } else {
-      maintainer_script_installed(deconpil->pkg, PRERMFILE, "pre-removal",
-                                  "deconfigure", "in-favour",
-                                  pkgbin_name(pkg, &pkg->available, pnaw_nonambig),
-                                  versiondescribe(&pkg->available.version,
-                                                  vdew_nonambig),
-                                  NULL);
+      maintscript_installed(deconpil->pkg, PRERMFILE, "pre-removal",
+                            "deconfigure", "in-favour",
+                            pkgbin_name(pkg, &pkg->available, pnaw_nonambig),
+                            versiondescribe(&pkg->available.version,
+                                            vdew_nonambig),
+                            NULL);
     }
   }
 
-  for (i = 0 ; i < cflict_index; i++) {
-    if (!(conflictor[i]->status == stat_halfconfigured ||
-          conflictor[i]->status == stat_triggersawaited ||
-          conflictor[i]->status == stat_triggerspending ||
-          conflictor[i]->status == stat_installed)) continue;
-    trig_activate_packageprocessing(conflictor[i]);
-    pkg_set_status(conflictor[i], stat_halfconfigured);
-    modstatdb_note(conflictor[i]);
+  for (conflictor_iter = conflictors.head;
+       conflictor_iter;
+       conflictor_iter = conflictor_iter->next) {
+    struct pkginfo *conflictor = conflictor_iter->pkg;
+
+    if (!(conflictor->status == PKG_STAT_HALFCONFIGURED ||
+          conflictor->status == PKG_STAT_TRIGGERSAWAITED ||
+          conflictor->status == PKG_STAT_TRIGGERSPENDING ||
+          conflictor->status == PKG_STAT_INSTALLED))
+      continue;
+
+    trig_activate_packageprocessing(conflictor);
+    pkg_set_status(conflictor, PKG_STAT_HALFCONFIGURED);
+    modstatdb_note(conflictor);
     push_cleanup(cu_prerminfavour, ~ehflag_normaltidy, NULL, 0,
-                 2,(void*)conflictor[i],(void*)pkg);
-    maintainer_script_installed(conflictor[i], PRERMFILE, "pre-removal",
-                                "remove", "in-favour",
-                                pkgbin_name(pkg, &pkg->available, pnaw_nonambig),
-                                versiondescribe(&pkg->available.version,
-                                                vdew_nonambig),
-                                NULL);
-    pkg_set_status(conflictor[i], stat_halfinstalled);
-    modstatdb_note(conflictor[i]);
+                 2, conflictor, pkg);
+    maintscript_installed(conflictor, PRERMFILE, "pre-removal",
+                          "remove", "in-favour",
+                          pkgbin_name(pkg, &pkg->available, pnaw_nonambig),
+                          versiondescribe(&pkg->available.version,
+                                          vdew_nonambig),
+                          NULL);
+    pkg_set_status(conflictor, PKG_STAT_HALFINSTALLED);
+    modstatdb_note(conflictor);
   }
 
-  pkg_set_eflags(pkg, eflag_reinstreq);
-  if (pkg->status == stat_notinstalled) {
+  pkg_set_eflags(pkg, PKG_EFLAG_REINSTREQ);
+  if (pkg->status == PKG_STAT_NOTINSTALLED) {
     pkg->installed.version= pkg->available.version;
     pkg->installed.multiarch = pkg->available.multiarch;
   }
-  pkg_set_status(pkg, stat_halfinstalled);
+  pkg_set_status(pkg, PKG_STAT_HALFINSTALLED);
   modstatdb_note(pkg);
-  if (oldversionstatus == stat_notinstalled) {
+  if (oldversionstatus == PKG_STAT_NOTINSTALLED) {
     push_cleanup(cu_preinstverynew, ~ehflag_normaltidy, NULL, 0,
                  3,(void*)pkg,(void*)cidir,(void*)cidirrest);
-    maintainer_script_new(pkg, PREINSTFILE, "pre-installation", cidir, cidirrest,
-                          "install", NULL);
-  } else if (oldversionstatus == stat_configfiles) {
+    maintscript_new(pkg, PREINSTFILE, "pre-installation", cidir, cidirrest,
+                    "install", NULL);
+  } else if (oldversionstatus == PKG_STAT_CONFIGFILES) {
     push_cleanup(cu_preinstnew, ~ehflag_normaltidy, NULL, 0,
                  3,(void*)pkg,(void*)cidir,(void*)cidirrest);
-    maintainer_script_new(pkg, PREINSTFILE, "pre-installation", cidir, cidirrest,
-                          "install", versiondescribe(&pkg->installed.version,
-                                                     vdew_nonambig),
-                          NULL);
+    maintscript_new(pkg, PREINSTFILE, "pre-installation", cidir, cidirrest,
+                    "install",
+                    versiondescribe(&pkg->installed.version, vdew_nonambig),
+                    NULL);
   } else {
     push_cleanup(cu_preinstupgrade, ~ehflag_normaltidy, NULL, 0,
                  4,(void*)pkg,(void*)cidir,(void*)cidirrest,(void*)&oldversionstatus);
-    maintainer_script_new(pkg, PREINSTFILE, "pre-installation", cidir, cidirrest,
-                          "upgrade", versiondescribe(&pkg->installed.version,
-                                                     vdew_nonambig),
-                          NULL);
-    printf(_("Unpacking replacement %.250s ...\n"),
-           pkgbin_name(pkg, &pkg->available, pnaw_nonambig));
+    maintscript_new(pkg, PREINSTFILE, "pre-installation", cidir, cidirrest,
+                    "upgrade",
+                    versiondescribe(&pkg->installed.version, vdew_nonambig),
+                    NULL);
+  }
+
+  if (oldversionstatus == PKG_STAT_NOTINSTALLED ||
+      oldversionstatus == PKG_STAT_CONFIGFILES) {
+    printf(_("Unpacking %s (%s) ...\n"),
+           pkgbin_name(pkg, &pkg->available, pnaw_nonambig),
+           versiondescribe(&pkg->available.version, vdew_nonambig));
+  } else {
+    printf(_("Unpacking %s (%s) over (%s) ...\n"),
+           pkgbin_name(pkg, &pkg->available, pnaw_nonambig),
+           versiondescribe(&pkg->available.version, vdew_nonambig),
+           versiondescribe(&pkg->installed.version, vdew_nonambig));
   }
 
   /*
@@ -921,8 +966,8 @@ void process_archive(const char *filename) {
   tc.backendpipe= p1[0];
   tc.pkgset_getting_in_sync = pkgset_getting_in_sync(pkg);
 
-  r = tar_extractor(&tc, &tf);
-  if (r) {
+  rc = tar_extractor(&tc, &tf);
+  if (rc) {
     if (errno) {
       ohshite(_("error reading dpkg-deb tar output"));
     } else {
@@ -933,18 +978,19 @@ void process_archive(const char *filename) {
     ohshit(_("cannot zap possible trailing zeros from dpkg-deb: %s"), err.str);
   close(p1[0]);
   p1[0] = -1;
-  subproc_wait_check(pid, BACKEND " --fsys-tarfile", PROCPIPE);
+  subproc_reap(pid, BACKEND " --fsys-tarfile", SUBPROC_NOPIPE);
 
   tar_deferred_extract(newfileslist, pkg);
 
-  if (oldversionstatus == stat_halfinstalled || oldversionstatus == stat_unpacked) {
+  if (oldversionstatus == PKG_STAT_HALFINSTALLED ||
+      oldversionstatus == PKG_STAT_UNPACKED) {
     /* Packages that were in ‘installed’ and ‘postinstfailed’ have been
      * reduced to ‘unpacked’ by now, by the running of the prerm script. */
-    pkg_set_status(pkg, stat_halfinstalled);
+    pkg_set_status(pkg, PKG_STAT_HALFINSTALLED);
     modstatdb_note(pkg);
     push_cleanup(cu_postrmupgrade, ~ehflag_normaltidy, NULL, 0, 1, (void *)pkg);
-    maintainer_script_alternative(pkg, POSTRMFILE, "post-removal", cidir, cidirrest,
-                                  "upgrade", "failed-upgrade");
+    maintscript_fallback(pkg, POSTRMFILE, "post-removal", cidir, cidirrest,
+                         "upgrade", "failed-upgrade");
   }
 
   /* If anything goes wrong while tidying up it's a bit late to do
@@ -975,7 +1021,6 @@ void process_archive(const char *filename) {
       continue;
 
     usenode = namenodetouse(namenode, pkg, &pkg->installed);
-    trig_file_activate(usenode, pkg);
 
     varbuf_trunc(&fnamevb, fnameidlu);
     varbuf_add_str(&fnamevb, usenode->name);
@@ -995,6 +1040,12 @@ void process_archive(const char *filename) {
       continue;
     }
     if (S_ISDIR(oldfs.st_mode)) {
+      trig_path_activate(usenode, pkg);
+
+      /* Do not try to remove the root directory. */
+      if (strcmp(usenode->name, "/.") == 0)
+        continue;
+
       if (rmdir(fnamevb.buf)) {
 	warning(_("unable to delete old directory '%.250s': %s"),
 	        namenode->name, strerror(errno));
@@ -1037,7 +1088,6 @@ void process_archive(const char *filename) {
 
 	  varbuf_reset(&cfilename);
 	  varbuf_add_str(&cfilename, instdir);
-	  varbuf_add_char(&cfilename, '/');
 	  varbuf_add_str(&cfilename, cfile->namenode->name);
 	  varbuf_end_str(&cfilename);
 
@@ -1086,7 +1136,7 @@ void process_archive(const char *filename) {
 	  debug(dbg_eachfile, "process_archive: old conff %s"
 		" is disappearing", namenode->name);
 	  namenode->flags |= fnnf_obs_conff;
-	  newconff_append(&newconffileslastp, namenode);
+	  filenamenode_queue_push(&newconffiles, namenode);
 	  addfiletolist(&tc, namenode);
 	}
 	continue;
@@ -1095,6 +1145,8 @@ void process_archive(const char *filename) {
       if (sameas)
 	continue;
 
+      trig_path_activate(usenode, pkg);
+
       if (secure_unlink_statted(fnamevb.buf, &oldfs)) {
         warning(_("unable to securely remove old file '%.250s': %s"),
                 namenode->name, strerror(errno));
@@ -1161,7 +1213,7 @@ void process_archive(const char *filename) {
       newpossi->arch_is_implicit = possi->arch_is_implicit;
       newpossi->arch = possi->arch;
       newpossi->verrel= possi->verrel;
-      if (possi->verrel != dpkg_relation_none)
+      if (possi->verrel != DPKG_RELATION_NONE)
         newpossi->version= possi->version;
       else
         dpkg_version_blank(&newpossi->version);
@@ -1195,7 +1247,7 @@ void process_archive(const char *filename) {
   /* We have to generate our own conffiles structure. */
   pkg->installed.conffiles = NULL;
   iconffileslastp = &pkg->installed.conffiles;
-  for (cfile= newconffiles; cfile; cfile= cfile->next) {
+  for (cfile = newconffiles.head; cfile; cfile = cfile->next) {
     newiconff= nfmalloc(sizeof(struct conffile));
     newiconff->next = NULL;
     newiconff->name= nfstrsave(cfile->namenode->name);
@@ -1220,7 +1272,7 @@ void process_archive(const char *filename) {
     if (otherpkg->installed.arch != pkg->installed.arch)
       continue;
 
-    assert(otherpkg->status == stat_notinstalled);
+    assert(otherpkg->status == PKG_STAT_NOTINSTALLED);
 
     pkg_blank(otherpkg);
   }
@@ -1238,20 +1290,20 @@ void process_archive(const char *filename) {
   while ((otherpkg = pkg_db_iter_next_pkg(it)) != NULL) {
     ensure_package_clientdata(otherpkg);
     if (otherpkg == pkg ||
-        otherpkg->status == stat_notinstalled ||
-        otherpkg->status == stat_configfiles ||
-	otherpkg->clientdata->istobe == itb_remove ||
+        otherpkg->status == PKG_STAT_NOTINSTALLED ||
+        otherpkg->status == PKG_STAT_CONFIGFILES ||
+        otherpkg->clientdata->istobe == PKG_ISTOBE_REMOVE ||
         !otherpkg->clientdata->files) continue;
     /* Do not try to disappear other packages from the same set
      * if they are Multi-Arch: same */
-    if (pkg->installed.multiarch == multiarch_same &&
-        otherpkg->installed.multiarch == multiarch_same &&
+    if (pkg->installed.multiarch == PKG_MULTIARCH_SAME &&
+        otherpkg->installed.multiarch == PKG_MULTIARCH_SAME &&
         otherpkg->set == pkg->set)
       continue;
     debug(dbg_veryverbose, "process_archive checking disappearance %s",
           pkg_name(otherpkg, pnaw_always));
-    assert(otherpkg->clientdata->istobe == itb_normal ||
-           otherpkg->clientdata->istobe == itb_deconfigure);
+    assert(otherpkg->clientdata->istobe == PKG_ISTOBE_NORMAL ||
+           otherpkg->clientdata->istobe == PKG_ISTOBE_DECONFIGURE);
     for (cfile= otherpkg->clientdata->files;
          cfile && strcmp(cfile->namenode->name, "/.") == 0;
          cfile= cfile->next);
@@ -1265,7 +1317,7 @@ void process_archive(const char *filename) {
     if (cfile) continue;
 
     /* So dependency things will give right answers ... */
-    otherpkg->clientdata->istobe= itb_remove;
+    otherpkg->clientdata->istobe = PKG_ISTOBE_REMOVE;
     debug(dbg_veryverbose, "process_archive disappear checking dependencies");
     for (pdep = otherpkg->set->depended.installed;
          pdep;
@@ -1300,7 +1352,7 @@ void process_archive(const char *filename) {
       }
     break_from_both_loops_at_once:;
     }
-    otherpkg->clientdata->istobe= itb_normal;
+    otherpkg->clientdata->istobe = PKG_ISTOBE_NORMAL;
     if (pdep) continue;
 
     /* No, we're disappearing it. This is the wrong time to go and
@@ -1378,7 +1430,7 @@ void process_archive(const char *filename) {
    * The only thing that we have left to do with it is remove
    * backup files, and we can leave the user to fix that if and when
    * it happens (we leave the reinstall required flag, of course). */
-  pkg_set_status(pkg, stat_unpacked);
+  pkg_set_status(pkg, PKG_STAT_UNPACKED);
   modstatdb_note(pkg);
 
   /* Now we delete all the backup files that we made when
@@ -1391,10 +1443,18 @@ void process_archive(const char *filename) {
    * They stay recorded as obsolete conffiles and will eventually
    * (if not taken over by another package) be forgotten. */
   for (cfile= newfileslist; cfile; cfile= cfile->next) {
+    struct filenamenode *usenode;
+
     if (cfile->namenode->flags & fnnf_new_conff) continue;
+
+    usenode = namenodetouse(cfile->namenode, pkg, &pkg->installed);
+
+    /* Do not try to remove backups for the root directory. */
+    if (strcmp(usenode->name, "/.") == 0)
+      continue;
+
     varbuf_trunc(&fnametmpvb, fnameidlu);
-    varbuf_add_str(&fnametmpvb,
-                   namenodetouse(cfile->namenode, pkg, &pkg->installed)->name);
+    varbuf_add_str(&fnametmpvb, usenode->name);
     varbuf_add_str(&fnametmpvb, DPKGTEMPEXT);
     varbuf_end_str(&fnametmpvb);
     ensure_pathname_nonexisting(fnametmpvb.buf);
@@ -1410,13 +1470,15 @@ void process_archive(const char *filename) {
    * The files list for the conflictor is still a little inconsistent in-core,
    * as we have not yet updated the filename->packages mappings; however,
    * the package->filenames mapping is. */
-  for (i = 0 ; i < cflict_index ; i++) {
+  while (!pkg_queue_is_empty(&conflictors)) {
+    struct pkginfo *conflictor = pkg_queue_pop(&conflictors);
+
     /* We need to have the most up-to-date info about which files are
      * what ... */
     ensure_allinstfiles_available();
-    removal_bulk(conflictor[i]);
+    removal_bulk(conflictor);
   }
 
   if (cipaction->arg_int == act_install)
-    add_to_queue(pkg);
+    enqueue_package(pkg);
 }
